@@ -1,22 +1,8 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  *  Copyright (C) 2009 Carlos Garcia Campos
  *  Copyright (C) 2004 Marco Pesenti Gritti
  *  Copyright © 2018 Christian Persch
- *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2, or (at your option)
- *  any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
  */
 
 #include "config.h"
@@ -59,12 +45,14 @@ struct _PpsDocumentPrivate {
 
 	gchar **page_labels;
 	PpsPageSize *page_sizes;
-	GMutex mutex;
+
+	GWeakRef *cached_pages;
+	/* the last page that was requested through pps_document_get_page
+	is saved, it optimizes cases where the same page is repeatedly queried */
+	PpsPage *last_page;
 };
 
 static guint64 _pps_document_get_size (const char *uri);
-static gchar *_pps_document_get_page_label (PpsDocument *document,
-                                            PpsPage *page);
 
 typedef struct _PpsDocumentPrivate PpsDocumentPrivate;
 
@@ -105,6 +93,9 @@ pps_document_finalize (GObject *object)
 	g_clear_pointer (&priv->page_sizes, g_free);
 	g_clear_pointer (&priv->page_labels, g_strfreev);
 
+	g_clear_object (&priv->last_page);
+	g_clear_pointer (&priv->cached_pages, g_free);
+
 	G_OBJECT_CLASS (pps_document_parent_class)->finalize (object);
 }
 
@@ -151,8 +142,6 @@ pps_document_init (PpsDocument *document)
 
 	/* Assume all pages are the same size until proven otherwise */
 	priv->uniform = TRUE;
-
-	g_mutex_init (&priv->mutex);
 }
 
 static void
@@ -186,8 +175,6 @@ pps_document_class_init (PpsDocumentClass *klass)
  *
  * You can monitor changes to the modification state by connecting to the
  * notify::modified signal on @document.
- *
- * Since: 3.28
  */
 gboolean
 pps_document_get_modified (PpsDocument *document)
@@ -204,8 +191,6 @@ pps_document_get_modified (PpsDocument *document)
  * @modified: a boolean value to set the document as modified or not.
  *
  * Set the @document modification state as @modified.
- *
- * Since: 3.28
  */
 void
 pps_document_set_modified (PpsDocument *document,
@@ -221,24 +206,9 @@ pps_document_set_modified (PpsDocument *document,
 }
 
 void
-pps_document_doc_mutex_lock (PpsDocument *document)
-{
-	PpsDocumentPrivate *priv = GET_PRIVATE (document);
-
-	g_mutex_lock (&priv->mutex);
-}
-
-void
-pps_document_doc_mutex_unlock (PpsDocument *document)
-{
-	PpsDocumentPrivate *priv = GET_PRIVATE (document);
-
-	g_mutex_unlock (&priv->mutex);
-}
-
-static void
 pps_document_setup_cache (PpsDocument *document)
 {
+	PpsDocumentClass *klass = PPS_DOCUMENT_GET_CLASS (document);
 	PpsDocumentPrivate *priv = GET_PRIVATE (document);
 	gboolean custom_page_labels = FALSE;
 	gint n_pages = pps_document_get_n_pages (document);
@@ -249,14 +219,20 @@ pps_document_setup_cache (PpsDocument *document)
 	 */
 	priv->cache_loaded = FALSE;
 
+	priv->cached_pages = g_new0 (GWeakRef, n_pages);
+
 	for (i = 0; i < n_pages; i++) {
-		PpsPage *page = pps_document_get_page (document, i);
+		PpsPage *page;
 		gdouble page_width = 0;
 		gdouble page_height = 0;
 		PpsPageSize *page_size;
-		gchar *page_label;
+		gchar *page_label = NULL;
 
-		pps_document_get_page_size (document, i, &page_width, &page_height);
+		g_weak_ref_init (&priv->cached_pages[i], NULL);
+
+		page = pps_document_get_page (document, i);
+
+		klass->get_page_size (document, page, &page_width, &page_height);
 
 		if (i == 0) {
 			priv->uniform_width = page_width;
@@ -297,7 +273,9 @@ pps_document_setup_cache (PpsDocument *document)
 				priv->min_height = page_height;
 		}
 
-		page_label = _pps_document_get_page_label (document, page);
+		if (klass->get_page_label)
+			page_label = klass->get_page_label (document, page);
+
 		if (page_label) {
 			if (!priv->page_labels)
 				priv->page_labels = g_new0 (gchar *, n_pages + 1);
@@ -327,71 +305,6 @@ pps_document_setup_cache (PpsDocument *document)
 	priv->cache_loaded = TRUE;
 }
 
-static void
-pps_document_ensure_cache (PpsDocument *document)
-{
-	PpsDocumentPrivate *priv = GET_PRIVATE (document);
-
-	if (!priv->cache_loaded) {
-		g_mutex_lock (&priv->mutex);
-		pps_document_setup_cache (document);
-		g_mutex_unlock (&priv->mutex);
-	}
-}
-
-/**
- * pps_document_load_full:
- * @document: a #PpsDocument
- * @uri: the document's URI
- * @flags: flags from #PpsDocumentLoadFlags
- * @error: a #GError location to store an error, or %NULL
- *
- * Loads @document from @uri.
- *
- * On failure, %FALSE is returned and @error is filled in.
- * If the document is encrypted, PPS_DEFINE_ERROR_ENCRYPTED is returned.
- * If the backend cannot load the specific document, PPS_DOCUMENT_ERROR_INVALID
- * is returned. Other errors are possible too, depending on the backend
- * used to load the document and the URI, e.g. #GIOError, #GFileError, and
- * #GConvertError.
- *
- * Returns: %TRUE on success, or %FALSE on failure.
- */
-gboolean
-pps_document_load_full (PpsDocument *document,
-                        const char *uri,
-                        PpsDocumentLoadFlags flags,
-                        GError **error)
-{
-	PpsDocumentClass *klass = PPS_DOCUMENT_GET_CLASS (document);
-	gboolean retval;
-	GError *err = NULL;
-	PpsDocumentPrivate *priv = GET_PRIVATE (document);
-
-	retval = klass->load (document, uri, &err);
-	if (!retval) {
-		if (err) {
-			g_propagate_error (error, err);
-		} else {
-			g_warning ("%s::PpsDocument::load returned FALSE but did not fill in @error; fix the backend!\n",
-			           G_OBJECT_TYPE_NAME (document));
-
-			/* So upper layers don't crash */
-			g_set_error_literal (error,
-			                     PPS_DOCUMENT_ERROR,
-			                     PPS_DOCUMENT_ERROR_INVALID,
-			                     "Internal error in backend");
-		}
-	} else {
-		if (!(flags & PPS_DOCUMENT_LOAD_FLAG_NO_CACHE))
-			pps_document_setup_cache (document);
-		priv->uri = g_strdup (uri);
-		priv->file_size = _pps_document_get_size (uri);
-	}
-
-	return retval;
-}
-
 /**
  * pps_document_load:
  * @document: a #PpsDocument
@@ -415,15 +328,40 @@ pps_document_load (PpsDocument *document,
                    const char *uri,
                    GError **error)
 {
-	return pps_document_load_full (document, uri,
-	                               PPS_DOCUMENT_LOAD_FLAG_NONE, error);
+	PpsDocumentClass *klass = PPS_DOCUMENT_GET_CLASS (document);
+	gboolean retval;
+	GError *err = NULL;
+	PpsDocumentPrivate *priv = GET_PRIVATE (document);
+	const gchar *uncompressed_uri;
+
+	uncompressed_uri = g_object_get_data (G_OBJECT (document),
+	                                      "uri-uncompressed");
+	retval = klass->load (document, uncompressed_uri ? uncompressed_uri : uri, &err);
+	if (!retval) {
+		if (err) {
+			g_propagate_error (error, err);
+		} else {
+			g_critical ("%s::PpsDocument::load returned FALSE but did not fill in @error; fix the backend!\n",
+			            G_OBJECT_TYPE_NAME (document));
+
+			/* So upper layers don't crash */
+			g_set_error_literal (error,
+			                     PPS_DOCUMENT_ERROR,
+			                     PPS_DOCUMENT_ERROR_INVALID,
+			                     "Internal error in backend");
+		}
+	} else {
+		priv->uri = g_strdup (uri);
+		priv->file_size = _pps_document_get_size (uri);
+	}
+
+	return retval;
 }
 
 /**
  * pps_document_load_fd:
  * @document: a #PpsDocument
  * @fd: a file descriptor
- * @flags: flags from #PpsDocumentLoadFlags
  * @error: (allow-none): a #GError location to store an error, or %NULL
  *
  * Synchronously loads the document from @fd, which must refer to
@@ -442,7 +380,6 @@ pps_document_load (PpsDocument *document,
 gboolean
 pps_document_load_fd (PpsDocument *document,
                       int fd,
-                      PpsDocumentLoadFlags flags,
                       GError **error)
 {
 	PpsDocumentClass *klass;
@@ -462,7 +399,7 @@ pps_document_load_fd (PpsDocument *document,
 	}
 
 	if (fstat (fd, &statbuf) == -1 ||
-	    (fd_flags = fcntl (fd, F_GETFL, &flags)) == -1) {
+	    (fd_flags = fcntl (fd, F_GETFL, NULL)) == -1) {
 		int errsv = errno;
 		g_set_error_literal (error, G_FILE_ERROR,
 		                     g_file_error_from_errno (errsv),
@@ -490,12 +427,9 @@ pps_document_load_fd (PpsDocument *document,
 		return FALSE;
 	}
 
-	if (!klass->load_fd (document, fd, flags, error))
+	if (!klass->load_fd (document, fd, error))
 		return FALSE;
 
-	if (!(flags & PPS_DOCUMENT_LOAD_FLAG_NO_CACHE))
-		pps_document_setup_cache (document);
-
 	return TRUE;
 }
 
@@ -531,8 +465,24 @@ pps_document_get_page (PpsDocument *document,
                        gint index)
 {
 	PpsDocumentClass *klass = PPS_DOCUMENT_GET_CLASS (document);
+	PpsDocumentPrivate *priv = GET_PRIVATE (document);
+	PpsPage *pps_page;
 
-	return klass->get_page (document, index);
+	/* Cache may not be loaded if this is called from the thumbnailer */
+	if (!priv->cache_loaded) {
+		pps_page = klass->get_page (document, index);
+		return pps_page;
+	}
+
+	if ((pps_page = g_weak_ref_get (&priv->cached_pages[index]))) {
+		return pps_page;
+	}
+
+	pps_page = klass->get_page (document, index);
+	g_weak_ref_set (&priv->cached_pages[index], pps_page);
+	g_set_object (&priv->last_page, pps_page);
+
+	return pps_page;
 }
 
 static guint64
@@ -577,59 +527,56 @@ pps_document_get_page_size (PpsDocument *document,
                             double *width,
                             double *height)
 {
-	PpsDocumentClass *klass = PPS_DOCUMENT_GET_CLASS (document);
-	PpsDocumentPrivate *priv;
+	PpsDocumentPrivate *priv = GET_PRIVATE (document);
 
 	g_return_if_fail (PPS_IS_DOCUMENT (document));
-	priv = GET_PRIVATE (document);
 	g_return_if_fail (0 <= page_index && page_index < pps_document_get_n_pages (document));
+	g_return_if_fail (priv->cache_loaded == TRUE);
 
-	if (priv->cache_loaded) {
-		if (width)
-			*width = priv->uniform ? priv->uniform_width : priv->page_sizes[page_index].width;
-		if (height)
-			*height = priv->uniform ? priv->uniform_height : priv->page_sizes[page_index].height;
-	} else {
-		PpsPage *page;
-
-		g_mutex_lock (&priv->mutex);
-		page = pps_document_get_page (document, page_index);
-		klass->get_page_size (document, page, width, height);
-		g_object_unref (page);
-		g_mutex_unlock (&priv->mutex);
-	}
+	if (width)
+		*width = priv->uniform ? priv->uniform_width : priv->page_sizes[page_index].width;
+	if (height)
+		*height = priv->uniform ? priv->uniform_height : priv->page_sizes[page_index].height;
 }
 
-static gchar *
-_pps_document_get_page_label (PpsDocument *document,
-                              PpsPage *page)
+/**
+ * pps_document_get_page_size_uncached:
+ * @document: a #PpsDocument
+ * @page: a #PpsPage
+ * @width: (out) (allow-none): return location for the width of the page, or %NULL
+ * @height: (out) (allow-none): return location for the height of the page, or %NULL
+ *
+ * Calls the @document's backend to fetch the page size. This should only be
+ * used on very specific cases, where it is not desired that the cache should
+ * be loaded. Calling with a loaded cache is a programmers' error.
+ *
+ * Since: 49.0
+ */
+void
+pps_document_get_page_size_uncached (PpsDocument *document,
+                                     PpsPage *page,
+                                     double *width,
+                                     double *height)
 {
+	PpsDocumentPrivate *priv = GET_PRIVATE (document);
 	PpsDocumentClass *klass = PPS_DOCUMENT_GET_CLASS (document);
 
-	return klass->get_page_label ? klass->get_page_label (document, page) : NULL;
+	g_return_if_fail (PPS_IS_DOCUMENT (document));
+	g_return_if_fail (PPS_IS_PAGE (page));
+	g_return_if_fail (priv->cache_loaded == FALSE);
+
+	return klass->get_page_size (document, page, width, height);
 }
 
 gchar *
 pps_document_get_page_label (PpsDocument *document,
                              gint page_index)
 {
-	PpsDocumentPrivate *priv;
+	PpsDocumentPrivate *priv = GET_PRIVATE (document);
+
 	g_return_val_if_fail (PPS_IS_DOCUMENT (document), NULL);
-	priv = GET_PRIVATE (document);
 	g_return_val_if_fail (0 <= page_index && page_index < pps_document_get_n_pages (document), NULL);
-
-	if (!priv->cache_loaded) {
-		PpsPage *page;
-		gchar *page_label;
-
-		g_mutex_lock (&priv->mutex);
-		page = pps_document_get_page (document, page_index);
-		page_label = _pps_document_get_page_label (document, page);
-		g_object_unref (page);
-		g_mutex_unlock (&priv->mutex);
-
-		return page_label ? page_label : g_strdup_printf ("%d", page_index + 1);
-	}
+	g_return_val_if_fail (priv->cache_loaded == TRUE, NULL);
 
 	return (priv->page_labels && priv->page_labels[page_index]) ? g_strdup (priv->page_labels[page_index]) : g_strdup_printf ("%d", page_index + 1);
 }
@@ -715,8 +662,6 @@ pps_document_get_thumbnail (PpsDocument *document,
  * @rc: an #PpsRenderContext
  *
  * Returns: (transfer full): a #cairo_surface_t
- *
- * Since: 3.14
  */
 cairo_surface_t *
 pps_document_get_thumbnail_surface (PpsDocument *document,
@@ -739,22 +684,28 @@ pps_document_get_uri (PpsDocument *document)
 	return priv->uri;
 }
 
+/**
+ * pps_document_get_title:
+ * @document: an #PpsDocument
+ *
+ * Returns: (transfer full):
+ */
 const gchar *
 pps_document_get_title (PpsDocument *document)
 {
 	g_return_val_if_fail (PPS_IS_DOCUMENT (document), NULL);
-	g_autofree PpsDocumentInfo *info = pps_document_get_info (document);
+	g_autoptr (PpsDocumentInfo) info = pps_document_get_info (document);
 
-	return (info->fields_mask & PPS_DOCUMENT_INFO_TITLE) ? info->title : NULL;
+	return (info->fields_mask & PPS_DOCUMENT_INFO_TITLE) ? g_strdup (info->title) : NULL;
 }
 
 gboolean
 pps_document_is_page_size_uniform (PpsDocument *document)
 {
-	g_return_val_if_fail (PPS_IS_DOCUMENT (document), TRUE);
 	PpsDocumentPrivate *priv = GET_PRIVATE (document);
 
-	pps_document_ensure_cache (document);
+	g_return_val_if_fail (PPS_IS_DOCUMENT (document), TRUE);
+	g_return_val_if_fail (priv->cache_loaded == TRUE, TRUE);
 
 	return priv->uniform;
 }
@@ -771,10 +722,10 @@ pps_document_get_max_page_size (PpsDocument *document,
                                 gdouble *width,
                                 gdouble *height)
 {
-	g_return_if_fail (PPS_IS_DOCUMENT (document));
 	PpsDocumentPrivate *priv = GET_PRIVATE (document);
 
-	pps_document_ensure_cache (document);
+	g_return_if_fail (PPS_IS_DOCUMENT (document));
+	g_return_if_fail (priv->cache_loaded == TRUE);
 
 	if (width)
 		*width = priv->max_width;
@@ -794,10 +745,10 @@ pps_document_get_min_page_size (PpsDocument *document,
                                 gdouble *width,
                                 gdouble *height)
 {
-	g_return_if_fail (PPS_IS_DOCUMENT (document));
 	PpsDocumentPrivate *priv = GET_PRIVATE (document);
 
-	pps_document_ensure_cache (document);
+	g_return_if_fail (PPS_IS_DOCUMENT (document));
+	g_return_if_fail (priv->cache_loaded == TRUE);
 
 	if (width)
 		*width = priv->min_width;
@@ -808,10 +759,10 @@ pps_document_get_min_page_size (PpsDocument *document,
 gboolean
 pps_document_check_dimensions (PpsDocument *document)
 {
-	g_return_val_if_fail (PPS_IS_DOCUMENT (document), FALSE);
 	PpsDocumentPrivate *priv = GET_PRIVATE (document);
 
-	pps_document_ensure_cache (document);
+	g_return_val_if_fail (PPS_IS_DOCUMENT (document), FALSE);
+	g_return_val_if_fail (priv->cache_loaded == TRUE, FALSE);
 
 	return (priv->max_width > 0 && priv->max_height > 0);
 }
@@ -828,10 +779,10 @@ pps_document_get_size (PpsDocument *document)
 gint
 pps_document_get_max_label_len (PpsDocument *document)
 {
-	g_return_val_if_fail (PPS_IS_DOCUMENT (document), -1);
 	PpsDocumentPrivate *priv = GET_PRIVATE (document);
 
-	pps_document_ensure_cache (document);
+	g_return_val_if_fail (PPS_IS_DOCUMENT (document), -1);
+	g_return_val_if_fail (priv->cache_loaded == TRUE, -1);
 
 	return priv->max_label;
 }
@@ -839,10 +790,10 @@ pps_document_get_max_label_len (PpsDocument *document)
 gboolean
 pps_document_has_text_page_labels (PpsDocument *document)
 {
-	g_return_val_if_fail (PPS_IS_DOCUMENT (document), FALSE);
 	PpsDocumentPrivate *priv = GET_PRIVATE (document);
 
-	pps_document_ensure_cache (document);
+	g_return_val_if_fail (PPS_IS_DOCUMENT (document), FALSE);
+	g_return_val_if_fail (priv->cache_loaded == TRUE, FALSE);
 
 	return priv->page_labels != NULL;
 }
@@ -869,8 +820,7 @@ pps_document_find_page_by_label (PpsDocument *document,
 	g_return_val_if_fail (PPS_IS_DOCUMENT (document), FALSE);
 	g_return_val_if_fail (page_label != NULL, FALSE);
 	g_return_val_if_fail (page_index != NULL, FALSE);
-
-	pps_document_ensure_cache (document);
+	g_return_val_if_fail (priv->cache_loaded == TRUE, FALSE);
 
 	n_pages = pps_document_get_n_pages (document);
 
@@ -931,20 +881,20 @@ pps_point_copy (PpsPoint *point)
 	return new_point;
 }
 
-/* PpsMark */
-G_DEFINE_BOXED_TYPE (PpsMark, pps_mark, pps_mark_copy, g_free)
+/* PpsDocumentPoint */
+G_DEFINE_BOXED_TYPE (PpsDocumentPoint, pps_document_point, pps_document_point_copy, g_free)
 
-PpsMark *
-pps_mark_copy (PpsMark *mark)
+PpsDocumentPoint *
+pps_document_point_copy (PpsDocumentPoint *document_point)
 {
-	PpsMark *new_mark;
+	PpsDocumentPoint *new_document_point;
 
-	g_return_val_if_fail (mark != NULL, NULL);
+	g_return_val_if_fail (document_point != NULL, NULL);
 
-	new_mark = g_new (PpsMark, 1);
-	*new_mark = *mark;
+	new_document_point = g_new (PpsDocumentPoint, 1);
+	*new_document_point = *document_point;
 
-	return new_mark;
+	return new_document_point;
 }
 
 /* PpsRectangle */
