@@ -241,10 +241,8 @@ get_page_text (PpsDocument *document,
 	g_autofree gchar *text = NULL;
 	gboolean success;
 
-	pps_document_doc_mutex_lock (document);
 	text = pps_document_text_get_text (PPS_DOCUMENT_TEXT (document), page);
 	success = pps_document_text_get_text_layout (PPS_DOCUMENT_TEXT (document), page, areas, n_areas);
-	pps_document_doc_mutex_unlock (document);
 
 	if (!success)
 		return NULL;
@@ -585,11 +583,19 @@ pps_search_context_set_search_term (PpsSearchContext *context,
 	if (g_strcmp0 (search_term, priv->search_term) == 0)
 		return;
 
+	if (priv->search_term)
+		g_free (priv->search_term);
+
 	priv->search_term = g_strdup (search_term);
 	g_object_notify_by_pspec (G_OBJECT (context), props[PROP_SEARCH_TERM]);
 	pps_search_context_restart (context);
 }
 
+/**
+ * pps_search_context_get_search_term:
+ *
+ * Returns: (transfer full)
+ */
 const gchar *
 pps_search_context_get_search_term (PpsSearchContext *context)
 {
@@ -656,6 +662,18 @@ pps_search_context_get_results_on_page (PpsSearchContext *context,
 	return g_ptr_array_copy (result_array, (GCopyFunc) g_object_ref, NULL);
 }
 
+gboolean
+pps_search_context_has_results_on_page (PpsSearchContext *context,
+                                        guint page)
+{
+	g_return_val_if_fail (PPS_IS_SEARCH_CONTEXT (context), false);
+
+	PpsSearchContextPrivate *priv = GET_PRIVATE (context);
+	GPtrArray *result_array = (GPtrArray *) g_hash_table_lookup (priv->per_page_store, GINT_TO_POINTER (page));
+
+	return result_array != NULL && result_array->len > 0;
+}
+
 /**
  * pps_search_context_activate:
  *
