@@ -1,21 +1,8 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /* this file is part of papers, a gnome document viewer
  *
  *  Copyright (C) 2008 Carlos Garcia Campos <carlosgc@gnome.org>
  *  Copyright (C) 2005 Red Hat, Inc
- *
- * Papers is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * Papers is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include <config.h>
@@ -119,22 +106,18 @@ fill_page_labels (GListModel *model, PpsJob *job)
 	}
 }
 
-static gboolean
+static void
 pps_job_links_run (PpsJob *job)
 {
 	PpsJobLinksPrivate *priv = JOB_LINKS_GET_PRIVATE (PPS_JOB_LINKS (job));
 
 	g_debug ("running links job");
 
-	pps_document_doc_mutex_lock (pps_job_get_document (job));
 	priv->model = pps_document_links_get_links_model (PPS_DOCUMENT_LINKS (pps_job_get_document (job)));
-	pps_document_doc_mutex_unlock (pps_job_get_document (job));
 
 	fill_page_labels (priv->model, job);
 
 	pps_job_succeeded (job);
-
-	return FALSE;
 }
 
 static void
@@ -168,8 +151,6 @@ pps_job_links_new (PpsDocument *document)
  * Get a #GListModel loaded with the links
  *
  * Returns: (nullable) (transfer none): a #GListModel of #PpsOutlines objects
- *
- * Since: 3.6
  */
 GListModel *
 pps_job_links_get_model (PpsJobLinks *self)
@@ -203,21 +184,17 @@ pps_job_attachments_dispose (GObject *object)
 	G_OBJECT_CLASS (pps_job_attachments_parent_class)->dispose (object);
 }
 
-static gboolean
+static void
 pps_job_attachments_run (PpsJob *job)
 {
 	PpsJobAttachmentsPrivate *priv = JOB_ATTACHMENTS_GET_PRIVATE (PPS_JOB_ATTACHMENTS (job));
 
 	g_debug ("running attachments job");
 
-	pps_document_doc_mutex_lock (pps_job_get_document (job));
 	priv->attachments =
 	    pps_document_attachments_get_attachments (PPS_DOCUMENT_ATTACHMENTS (pps_job_get_document (job)));
-	pps_document_doc_mutex_unlock (pps_job_get_document (job));
 
 	pps_job_succeeded (job);
-
-	return FALSE;
 }
 
 /**
@@ -282,14 +259,13 @@ pps_job_annots_dispose (GObject *object)
 	G_OBJECT_CLASS (pps_job_annots_parent_class)->dispose (object);
 }
 
-static gboolean
+static void
 pps_job_annots_run (PpsJob *job)
 {
 	PpsJobAnnotsPrivate *priv = JOB_ANNOTS_GET_PRIVATE (PPS_JOB_ANNOTS (job));
 
 	g_debug ("running annots job");
 
-	pps_document_doc_mutex_lock (pps_job_get_document (job));
 	for (gint i = 0; i < pps_document_get_n_pages (pps_job_get_document (job)); i++) {
 		GList *annots;
 		PpsPage *page;
@@ -302,11 +278,8 @@ pps_job_annots_run (PpsJob *job)
 		if (annots)
 			priv->annots = g_list_concat (priv->annots, annots);
 	}
-	pps_document_doc_mutex_unlock (pps_job_get_document (job));
 
 	pps_job_succeeded (job);
-
-	return FALSE;
 }
 
 static void
@@ -394,7 +367,7 @@ pps_job_render_texture_new (PpsDocument *document,
 	return PPS_JOB (job);
 }
 
-static gboolean
+static void
 pps_job_render_texture_run (PpsJob *job)
 {
 	PpsJobRenderTexture *job_render = PPS_JOB_RENDER_TEXTURE (job);
@@ -404,8 +377,6 @@ pps_job_render_texture_run (PpsJob *job)
 
 	g_debug ("running render job: page: %d (%p)", job_render->page, job);
 
-	pps_document_doc_mutex_lock (pps_job_get_document (job));
-
 	PPS_PROFILER_START (PPS_GET_TYPE_NAME (job), g_strdup_printf ("page: %d", job_render->page));
 
 	pps_page = pps_document_get_page (pps_job_get_document (job), job_render->page);
@@ -418,7 +389,6 @@ pps_job_render_texture_run (PpsJob *job)
 
 	if (surface == NULL ||
 	    cairo_surface_status (surface) != CAIRO_STATUS_SUCCESS) {
-		pps_document_doc_mutex_unlock (pps_job_get_document (job));
 		g_object_unref (rc);
 
 		if (surface != NULL) {
@@ -438,7 +408,7 @@ pps_job_render_texture_run (PpsJob *job)
 		}
 
 		job_render->texture = NULL;
-		return FALSE;
+		return;
 	}
 
 	job_render->texture = pps_document_misc_texture_from_surface (surface);
@@ -449,9 +419,8 @@ pps_job_render_texture_run (PpsJob *job)
 	 */
 	if (g_cancellable_is_cancelled (pps_job_get_cancellable (job))) {
 		PPS_PROFILER_STOP ();
-		pps_document_doc_mutex_unlock (pps_job_get_document (job));
 		g_object_unref (rc);
-		return FALSE;
+		return;
 	}
 
 	if (job_render->include_selection && PPS_IS_SELECTION (pps_job_get_document (job))) {
@@ -477,11 +446,8 @@ pps_job_render_texture_run (PpsJob *job)
 	g_object_unref (rc);
 
 	PPS_PROFILER_STOP ();
-	pps_document_doc_mutex_unlock (pps_job_get_document (job));
 
 	pps_job_succeeded (job);
-
-	return FALSE;
 }
 
 static void
@@ -515,7 +481,7 @@ pps_job_page_data_init (PpsJobPageData *job)
 {
 }
 
-static gboolean
+static void
 pps_job_page_data_run (PpsJob *job)
 {
 	PpsJobPageData *job_pd = PPS_JOB_PAGE_DATA (job);
@@ -523,7 +489,6 @@ pps_job_page_data_run (PpsJob *job)
 
 	g_debug ("running page data job: page: %d (%p)", job_pd->page, job);
 
-	pps_document_doc_mutex_lock (pps_job_get_document (job));
 	pps_page = pps_document_get_page (pps_job_get_document (job), job_pd->page);
 
 	if ((job_pd->flags & PPS_PAGE_DATA_INCLUDE_TEXT_MAPPING) && PPS_IS_DOCUMENT_TEXT (pps_job_get_document (job)))
@@ -559,20 +524,13 @@ pps_job_page_data_run (PpsJob *job)
 		job_pd->image_mapping =
 		    pps_document_images_get_image_mapping (PPS_DOCUMENT_IMAGES (pps_job_get_document (job)),
 		                                           pps_page);
-	if ((job_pd->flags & PPS_PAGE_DATA_INCLUDE_ANNOTS) && PPS_IS_DOCUMENT_ANNOTATIONS (pps_job_get_document (job)))
-		job_pd->annot_mapping =
-		    pps_document_annotations_get_annotations_mapping (PPS_DOCUMENT_ANNOTATIONS (pps_job_get_document (job)),
-		                                                      pps_page);
 	if ((job_pd->flags & PPS_PAGE_DATA_INCLUDE_MEDIA) && PPS_IS_DOCUMENT_MEDIA (pps_job_get_document (job)))
 		job_pd->media_mapping =
 		    pps_document_media_get_media_mapping (PPS_DOCUMENT_MEDIA (pps_job_get_document (job)),
 		                                          pps_page);
 	g_object_unref (pps_page);
-	pps_document_doc_mutex_unlock (pps_job_get_document (job));
 
 	pps_job_succeeded (job);
-
-	return FALSE;
 }
 
 static void
@@ -635,7 +593,7 @@ pps_job_thumbnail_texture_dispose (GObject *object)
 	G_OBJECT_CLASS (pps_job_thumbnail_texture_parent_class)->dispose (object);
 }
 
-static gboolean
+static void
 pps_job_thumbnail_texture_run (PpsJob *job)
 {
 	PpsJobThumbnailTexturePrivate *priv =
@@ -646,8 +604,6 @@ pps_job_thumbnail_texture_run (PpsJob *job)
 
 	g_debug ("running thumbnail job: page: %d (%p)", priv->page, job);
 
-	pps_document_doc_mutex_lock (pps_job_get_document (job));
-
 	PPS_PROFILER_START (PPS_GET_TYPE_NAME (job), g_strdup_printf ("page: %d", priv->page));
 	page = pps_document_get_page (pps_job_get_document (job), priv->page);
 	rc = pps_render_context_new (page, priv->rotation, priv->scale, PPS_RENDER_ANNOTS_ALL);
@@ -661,7 +617,6 @@ pps_job_thumbnail_texture_run (PpsJob *job)
 	cairo_surface_destroy (surface);
 	g_object_unref (rc);
 	PPS_PROFILER_STOP ();
-	pps_document_doc_mutex_unlock (pps_job_get_document (job));
 
 	if (priv->thumbnail_texture == NULL) {
 		pps_job_failed (job,
@@ -672,8 +627,6 @@ pps_job_thumbnail_texture_run (PpsJob *job)
 	} else {
 		pps_job_succeeded (job);
 	}
-
-	return FALSE;
 }
 
 static void
@@ -755,22 +708,16 @@ pps_job_fonts_init (PpsJobFonts *job)
 {
 }
 
-static gboolean
+static void
 pps_job_fonts_run (PpsJob *job)
 {
 	PpsDocument *document = pps_job_get_document (job);
 
 	g_debug ("running fonts job");
 
-	pps_document_doc_mutex_lock (pps_job_get_document (job));
-
 	pps_document_fonts_scan (PPS_DOCUMENT_FONTS (document));
 
-	pps_document_doc_mutex_unlock (pps_job_get_document (job));
-
 	pps_job_succeeded (job);
-
-	return FALSE;
 }
 
 static void
@@ -811,8 +758,8 @@ typedef struct _PpsJobLoadPrivate {
 	char *mime_type;
 	gchar *password;
 	GPasswordSave password_save;
-	PpsDocumentLoadFlags flags;
 	PpsDocument *loaded_document;
+	PpsDocument *document;
 } PpsJobLoadPrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE (PpsJobLoad, pps_job_load, PPS_TYPE_JOB)
@@ -842,6 +789,7 @@ pps_job_load_dispose (GObject *object)
 	g_clear_pointer (&priv->uri, g_free);
 	g_clear_pointer (&priv->password, g_free);
 	g_clear_object (&priv->loaded_document);
+	g_clear_object (&priv->document);
 
 	G_OBJECT_CLASS (pps_job_load_parent_class)->dispose (object);
 }
@@ -862,7 +810,7 @@ pps_dupfd (int fd,
 	return new_fd;
 }
 
-static gboolean
+static void
 pps_job_load_run (PpsJob *job)
 {
 	PpsJobLoad *job_load = PPS_JOB_LOAD (job);
@@ -876,62 +824,58 @@ pps_job_load_run (PpsJob *job)
 		                     "Either the URI or the FD must be set!");
 		pps_job_failed_from_error (job, error);
 		g_error_free (error);
-		return FALSE;
+		return;
 	}
 
 	/* This job may already have a document even if the job didn't complete
 	   because, e.g., a password is required - if so, just reload rather than
 	   creating a new instance */
-	if (priv->loaded_document) {
-		PpsDocument *loaded_doc = priv->loaded_document;
-		const gchar *uncompressed_uri;
-
+	if (priv->document) {
 		if (priv->password) {
-			pps_document_security_set_password (PPS_DOCUMENT_SECURITY (loaded_doc),
+			pps_document_security_set_password (PPS_DOCUMENT_SECURITY (priv->document),
 			                                    priv->password);
 		}
 
 		pps_job_reset (job);
 
 		if (priv->uri) {
-			uncompressed_uri = g_object_get_data (G_OBJECT (loaded_doc),
-			                                      "uri-uncompressed");
-			pps_document_load_full (loaded_doc,
-			                        uncompressed_uri ? uncompressed_uri : priv->uri,
-			                        priv->flags,
-			                        &error);
+			if (pps_document_load (priv->document, priv->uri, &error))
+				g_set_object (&priv->loaded_document, priv->document);
 		} else {
 			/* We need to dup the FD since we may need to pass it again
 			 * if the document is reloaded, as pps_document calls
 			 * consume it.
 			 */
 			int fd = pps_dupfd (priv->fd, &error);
-			if (fd != -1)
-				pps_document_load_fd (loaded_doc,
-				                      fd,
-				                      priv->flags,
-				                      &error);
+			if (fd != -1 && pps_document_load_fd (priv->document, fd, &error))
+				g_set_object (&priv->loaded_document, priv->document);
 		}
 	} else {
 		if (priv->uri) {
-			priv->loaded_document =
-			    pps_document_factory_get_document_full (priv->uri,
-			                                            priv->flags,
-			                                            &error);
+			priv->document = pps_document_factory_get_document (priv->uri, &error);
+			if (priv->document != NULL &&
+			    pps_document_load (priv->document, priv->uri, &error))
+				priv->loaded_document = g_object_ref (priv->document);
+
 		} else {
-			/* We need to dup the FD since we may need to pass it again
-			 * if the document is reloaded, as pps_document calls
-			 * consume it.
-			 */
-			int fd = pps_dupfd (priv->fd, &error);
-			if (fd != -1)
-				priv->loaded_document =
-				    pps_document_factory_get_document_for_fd (fd,
-				                                              priv->mime_type,
-				                                              priv->flags,
-				                                              &error);
+			priv->document = pps_document_factory_get_document_for_fd (priv->fd,
+			                                                           priv->mime_type,
+			                                                           &error);
+			if (priv->document != NULL) {
+				/* We need to dup the FD since we may need to pass it again
+				 * if the document is reloaded, as pps_document_load calls
+				 * consume it.
+				 */
+				int fd = pps_dupfd (priv->fd, &error);
+				if (fd != -1 && pps_document_load_fd (priv->document,
+				                                      fd,
+				                                      &error))
+					priv->loaded_document = g_object_ref (priv->document);
+			}
 		}
 	}
+	if (priv->document && priv->loaded_document)
+		pps_document_setup_cache (priv->document);
 
 	if (error) {
 		pps_job_failed_from_error (job, error);
@@ -939,8 +883,6 @@ pps_job_load_run (PpsJob *job)
 	} else {
 		pps_job_succeeded (job);
 	}
-
-	return FALSE;
 }
 
 static void
@@ -1109,16 +1051,6 @@ pps_job_load_get_password_save (PpsJobLoad *job)
 	return priv->password_save;
 }
 
-void
-pps_job_load_set_load_flags (PpsJobLoad *job,
-                             PpsDocumentLoadFlags flags)
-{
-	PpsJobLoadPrivate *priv = JOB_LOAD_GET_PRIVATE (job);
-	g_return_if_fail (PPS_IS_JOB_LOAD (job));
-
-	priv->flags = flags;
-}
-
 /**
  * pps_job_load_get_loaded_document:
  * @job: an #PpsJobLoad
@@ -1169,7 +1101,7 @@ pps_job_save_dispose (GObject *object)
 	G_OBJECT_CLASS (pps_job_save_parent_class)->dispose (object);
 }
 
-static gboolean
+static void
 pps_job_save_run (PpsJob *job)
 {
 	PpsJobSave *job_save = PPS_JOB_SAVE (job);
@@ -1185,23 +1117,19 @@ pps_job_save_run (PpsJob *job)
 	fd = pps_mkstemp ("saveacopy.XXXXXX", &tmp_filename, &error);
 	if (fd == -1) {
 		pps_job_failed_from_error (job, error);
-		return FALSE;
+		return;
 	}
 	close (fd);
 
-	pps_document_doc_mutex_lock (pps_job_get_document (job));
-
 	/* Save document to temp filename */
 	tmp_uri = g_filename_to_uri (tmp_filename, NULL, &error);
 	if (tmp_uri != NULL) {
 		pps_document_save (pps_job_get_document (job), tmp_uri, &error);
 	}
 
-	pps_document_doc_mutex_unlock (pps_job_get_document (job));
-
 	if (error) {
 		pps_job_failed_from_error (job, error);
-		return FALSE;
+		return;
 	}
 
 	/* If original document was compressed,
@@ -1232,11 +1160,11 @@ pps_job_save_run (PpsJob *job)
 
 	if (error) {
 		pps_job_failed_from_error (job, error);
-		return FALSE;
+		return;
 	}
 
 	if (!tmp_uri)
-		return FALSE;
+		return;
 
 	pps_xfer_uri_simple (tmp_uri, priv->uri, &error);
 	pps_tmp_uri_unlink (tmp_uri);
@@ -1252,8 +1180,6 @@ pps_job_save_run (PpsJob *job)
 	} else {
 		pps_job_succeeded (job);
 	}
-
-	return FALSE;
 }
 
 static void
@@ -1322,7 +1248,7 @@ pps_job_find_dispose (GObject *object)
 	(*G_OBJECT_CLASS (pps_job_find_parent_class)->dispose) (object);
 }
 
-static gboolean
+static void
 pps_job_find_run (PpsJob *job)
 {
 	PpsJobFind *job_find = PPS_JOB_FIND (job);
@@ -1338,14 +1264,12 @@ pps_job_find_run (PpsJob *job)
 
 	while (n_pages-- > 0) {
 		if (g_cancellable_is_cancelled (pps_job_get_cancellable (job)))
-			return FALSE;
+			return;
 
-		pps_document_doc_mutex_lock (pps_job_get_document (job));
 		pps_page = pps_document_get_page (pps_job_get_document (job), current_page);
 		matches = pps_document_find_find_text (find, pps_page, job_find->text,
 		                                       job_find->options);
 		g_object_unref (pps_page);
-		pps_document_doc_mutex_unlock (pps_job_get_document (job));
 
 		job_find->has_results |= (matches != NULL);
 
@@ -1356,8 +1280,6 @@ pps_job_find_run (PpsJob *job)
 	}
 
 	pps_job_succeeded (job);
-
-	return FALSE;
 }
 
 static void
@@ -1410,8 +1332,6 @@ pps_job_find_new (PpsDocument *document,
  * @job:
  *
  * Returns: the job's find options
- *
- * Since: 3.6
  */
 PpsFindOptions
 pps_job_find_get_options (PpsJobFind *job)
@@ -1486,20 +1406,16 @@ pps_job_layers_dispose (GObject *object)
 	G_OBJECT_CLASS (pps_job_layers_parent_class)->dispose (object);
 }
 
-static gboolean
+static void
 pps_job_layers_run (PpsJob *job)
 {
 	PpsJobLayersPrivate *priv = JOB_LAYERS_GET_PRIVATE (PPS_JOB_LAYERS (job));
 
 	g_debug ("running layers job");
 
-	pps_document_doc_mutex_lock (pps_job_get_document (job));
 	priv->model = pps_document_layers_get_layers (PPS_DOCUMENT_LAYERS (pps_job_get_document (job)));
-	pps_document_doc_mutex_unlock (pps_job_get_document (job));
 
 	pps_job_succeeded (job);
-
-	return FALSE;
 }
 
 static void
@@ -1568,7 +1484,7 @@ pps_job_export_dispose (GObject *object)
 	G_OBJECT_CLASS (pps_job_export_parent_class)->dispose (object);
 }
 
-static gboolean
+static void
 pps_job_export_run (PpsJob *job)
 {
 	PpsJobExportPrivate *priv = JOB_EXPORT_GET_PRIVATE (PPS_JOB_EXPORT (job));
@@ -1578,8 +1494,6 @@ pps_job_export_run (PpsJob *job)
 
 	g_debug ("running export job");
 
-	pps_document_doc_mutex_lock (pps_job_get_document (job));
-
 	pps_page = pps_document_get_page (pps_job_get_document (job), priv->page);
 	if (priv->rc) {
 		pps_job_reset (job);
@@ -1592,11 +1506,7 @@ pps_job_export_run (PpsJob *job)
 
 	pps_file_exporter_do_page (PPS_FILE_EXPORTER (pps_job_get_document (job)), priv->rc);
 
-	pps_document_doc_mutex_unlock (pps_job_get_document (job));
-
 	pps_job_succeeded (job);
-
-	return FALSE;
 }
 
 static void
@@ -1660,7 +1570,7 @@ pps_job_print_dispose (GObject *object)
 	G_OBJECT_CLASS (pps_job_print_parent_class)->dispose (object);
 }
 
-static gboolean
+static void
 pps_job_print_run (PpsJob *job)
 {
 	PpsJobPrintPrivate *priv = JOB_PRINT_GET_PRIVATE (PPS_JOB_PRINT (job));
@@ -1674,17 +1584,13 @@ pps_job_print_run (PpsJob *job)
 
 	pps_job_reset (job);
 
-	pps_document_doc_mutex_lock (pps_job_get_document (job));
-
 	pps_page = pps_document_get_page (pps_job_get_document (job), priv->page);
 	pps_document_print_print_page (PPS_DOCUMENT_PRINT (pps_job_get_document (job)),
 	                               pps_page, priv->cr);
 	g_object_unref (pps_page);
 
-	pps_document_doc_mutex_unlock (pps_job_get_document (job));
-
 	if (g_cancellable_is_cancelled (pps_job_get_cancellable (job)))
-		return FALSE;
+		return;
 
 	cr_status = cairo_status (priv->cr);
 	if (cr_status == CAIRO_STATUS_SUCCESS) {
@@ -1697,8 +1603,6 @@ pps_job_print_run (PpsJob *job)
 		                priv->page,
 		                cairo_status_to_string (cr_status));
 	}
-
-	return FALSE;
 }
 
 static void
@@ -1772,21 +1676,17 @@ pps_job_signatures_dispose (GObject *object)
 	G_OBJECT_CLASS (pps_job_signatures_parent_class)->dispose (object);
 }
 
-static gboolean
+static void
 pps_job_signatures_run (PpsJob *job)
 {
 	PpsJobSignaturesPrivate *priv = JOB_SIGNATURES_GET_PRIVATE (PPS_JOB_SIGNATURES (job));
 
 	g_debug ("running signatures job");
 
-	pps_document_doc_mutex_lock (pps_job_get_document (job));
 	priv->signatures =
 	    pps_document_signatures_get_signatures (PPS_DOCUMENT_SIGNATURES ((pps_job_get_document (job))));
-	pps_document_doc_mutex_unlock (pps_job_get_document (job));
 
 	pps_job_succeeded (job);
-
-	return FALSE;
 }
 
 static void
