@@ -1,20 +1,7 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /* this file is part of papers, a gnome document viewer
  *
  *  Copyright (C) 2009 Carlos Garcia Campos
- *
- * Papers is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * Papers is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include <config.h>
@@ -34,13 +21,11 @@ static guint pps_page_cache_signals[LAST_SIGNAL] = { 0 };
 typedef struct _PpsPageCacheData {
 	PpsJob *job;
 	gboolean done : 1;
-	gboolean dirty : 1;
 	PpsJobPageDataFlags flags;
 
 	PpsMappingList *link_mapping;
 	PpsMappingList *image_mapping;
 	PpsMappingList *form_field_mapping;
-	PpsMappingList *annot_mapping;
 	PpsMappingList *media_mapping;
 	cairo_region_t *text_mapping;
 	PpsRectangle *text_layout;
@@ -70,7 +55,6 @@ struct _PpsPageCache {
     PPS_PAGE_DATA_INCLUDE_TEXT_MAPPING | \
     PPS_PAGE_DATA_INCLUDE_IMAGES |       \
     PPS_PAGE_DATA_INCLUDE_FORMS |        \
-    PPS_PAGE_DATA_INCLUDE_ANNOTS |       \
     PPS_PAGE_DATA_INCLUDE_MEDIA)
 
 #define PRE_CACHE_SIZE 1
@@ -90,7 +74,6 @@ pps_page_cache_data_free (PpsPageCacheData *data)
 	g_clear_pointer (&data->link_mapping, pps_mapping_list_unref);
 	g_clear_pointer (&data->image_mapping, pps_mapping_list_unref);
 	g_clear_pointer (&data->form_field_mapping, pps_mapping_list_unref);
-	g_clear_pointer (&data->annot_mapping, pps_mapping_list_unref);
 	g_clear_pointer (&data->media_mapping, pps_mapping_list_unref);
 	g_clear_pointer (&data->text_mapping, cairo_region_destroy);
 
@@ -166,10 +149,9 @@ pps_page_cache_get_flags_for_data (PpsPageCache *cache,
 {
 	PpsJobPageDataFlags flags = PPS_PAGE_DATA_INCLUDE_NONE;
 
-	if (data->flags == cache->flags && !data->dirty)
+	if (data->flags == cache->flags)
 		return cache->flags;
 
-	/* Flags changed or data is dirty */
 	if (cache->flags & PPS_PAGE_DATA_INCLUDE_LINKS) {
 		flags = (data->link_mapping) ? flags & ~PPS_PAGE_DATA_INCLUDE_LINKS : flags | PPS_PAGE_DATA_INCLUDE_LINKS;
 	}
@@ -182,10 +164,6 @@ pps_page_cache_get_flags_for_data (PpsPageCache *cache,
 		flags = (data->form_field_mapping) ? flags & ~PPS_PAGE_DATA_INCLUDE_FORMS : flags | PPS_PAGE_DATA_INCLUDE_FORMS;
 	}
 
-	if (cache->flags & PPS_PAGE_DATA_INCLUDE_ANNOTS) {
-		flags = (data->annot_mapping) ? flags & ~PPS_PAGE_DATA_INCLUDE_ANNOTS : flags | PPS_PAGE_DATA_INCLUDE_ANNOTS;
-	}
-
 	if (cache->flags & PPS_PAGE_DATA_INCLUDE_MEDIA) {
 		flags = (data->media_mapping) ? flags & ~PPS_PAGE_DATA_INCLUDE_MEDIA : flags | PPS_PAGE_DATA_INCLUDE_MEDIA;
 	}
@@ -244,8 +222,6 @@ job_page_data_finished_cb (PpsJob *job,
 		data->image_mapping = job_data->image_mapping;
 	if (job_data->flags & PPS_PAGE_DATA_INCLUDE_FORMS)
 		data->form_field_mapping = job_data->form_field_mapping;
-	if (job_data->flags & PPS_PAGE_DATA_INCLUDE_ANNOTS)
-		data->annot_mapping = job_data->annot_mapping;
 	if (job_data->flags & PPS_PAGE_DATA_INCLUDE_MEDIA)
 		data->media_mapping = job_data->media_mapping;
 	if (job_data->flags & PPS_PAGE_DATA_INCLUDE_TEXT_MAPPING)
@@ -264,7 +240,6 @@ job_page_data_finished_cb (PpsJob *job,
 	}
 
 	data->done = TRUE;
-	data->dirty = FALSE;
 
 	g_clear_object (&data->job);
 
@@ -285,7 +260,7 @@ pps_page_cache_schedule_job_if_needed (PpsPageCache *cache,
 	PpsPageCacheData *data = &cache->page_list[page];
 	PpsJobPageDataFlags flags;
 
-	if (data->flags == cache->flags && !data->dirty && (data->done || data->job))
+	if (data->flags == cache->flags && (data->done || data->job))
 		return;
 
 	if (data->job)
@@ -358,56 +333,6 @@ pps_page_cache_set_flags (PpsPageCache *cache,
 	pps_page_cache_set_page_range (cache, cache->start_page, cache->end_page);
 }
 
-void
-pps_page_cache_mark_dirty (PpsPageCache *cache,
-                           gint page,
-                           PpsJobPageDataFlags flags)
-{
-	PpsPageCacheData *data;
-
-	g_return_if_fail (PPS_IS_PAGE_CACHE (cache));
-
-	data = &cache->page_list[page];
-	data->dirty = TRUE;
-
-	if (flags & PPS_PAGE_DATA_INCLUDE_LINKS)
-		g_clear_pointer (&data->link_mapping, pps_mapping_list_unref);
-
-	if (flags & PPS_PAGE_DATA_INCLUDE_IMAGES)
-		g_clear_pointer (&data->image_mapping, pps_mapping_list_unref);
-
-	if (flags & PPS_PAGE_DATA_INCLUDE_FORMS)
-		g_clear_pointer (&data->form_field_mapping, pps_mapping_list_unref);
-
-	if (flags & PPS_PAGE_DATA_INCLUDE_ANNOTS)
-		g_clear_pointer (&data->annot_mapping, pps_mapping_list_unref);
-
-	if (flags & PPS_PAGE_DATA_INCLUDE_MEDIA)
-		g_clear_pointer (&data->media_mapping, pps_mapping_list_unref);
-
-	if (flags & PPS_PAGE_DATA_INCLUDE_TEXT_MAPPING)
-		g_clear_pointer (&data->text_mapping, cairo_region_destroy);
-
-	if (flags & PPS_PAGE_DATA_INCLUDE_TEXT)
-		g_clear_pointer (&data->text, g_free);
-
-	if (flags & PPS_PAGE_DATA_INCLUDE_TEXT_LAYOUT) {
-		g_clear_pointer (&data->text_layout, g_free);
-		data->text_layout_length = 0;
-	}
-
-	if (flags & PPS_PAGE_DATA_INCLUDE_TEXT_ATTRS)
-		g_clear_pointer (&data->text_attrs, pango_attr_list_unref);
-
-	if (flags & PPS_PAGE_DATA_INCLUDE_TEXT_LOG_ATTRS) {
-		g_clear_pointer (&data->text_log_attrs, g_free);
-		data->text_log_attrs_length = 0;
-	}
-
-	/* Update the current range */
-	pps_page_cache_set_page_range (cache, cache->start_page, cache->end_page);
-}
-
 PpsMappingList *
 pps_page_cache_get_link_mapping (PpsPageCache *cache,
                                  gint page)
@@ -474,28 +399,6 @@ pps_page_cache_get_form_field_mapping (PpsPageCache *cache,
 	return data->form_field_mapping;
 }
 
-PpsMappingList *
-pps_page_cache_get_annot_mapping (PpsPageCache *cache,
-                                  gint page)
-{
-	PpsPageCacheData *data;
-
-	g_return_val_if_fail (PPS_IS_PAGE_CACHE (cache), NULL);
-	g_return_val_if_fail (page >= 0 && page < cache->n_pages, NULL);
-
-	if (!(cache->flags & PPS_PAGE_DATA_INCLUDE_ANNOTS))
-		return NULL;
-
-	data = &cache->page_list[page];
-	if (data->done)
-		return data->annot_mapping;
-
-	if (data->job)
-		return PPS_JOB_PAGE_DATA (data->job)->annot_mapping;
-
-	return data->annot_mapping;
-}
-
 PpsMappingList *
 pps_page_cache_get_media_mapping (PpsPageCache *cache,
                                   gint page)
@@ -600,8 +503,6 @@ pps_page_cache_get_text_layout (PpsPageCache *cache,
  * @page:
  *
  * FIXME
- *
- * Since: 3.10
  */
 PangoAttrList *
 pps_page_cache_get_text_attrs (PpsPageCache *cache,
@@ -635,8 +536,6 @@ pps_page_cache_get_text_attrs (PpsPageCache *cache,
  * FIXME
  *
  * Returns: %TRUE on success with @log_attrs filled in, %FALSE otherwise
- *
- * Since: 3.10
  */
 gboolean
 pps_page_cache_get_text_log_attrs (PpsPageCache *cache,
