@@ -483,6 +483,8 @@ impl imp::PpsDocumentView {
         let mut markup_type = "none";
         let has_selection = self.view.has_selection();
 
+        self.annot.replace(annot.cloned());
+
         if let Some(annot) = annot {
             show_annot_props = annot.is::<papers_document::AnnotationMarkup>();
             is_annot_textmarkup = annot.is::<papers_document::AnnotationTextMarkup>();
@@ -532,8 +534,6 @@ impl imp::PpsDocumentView {
             self.set_action_state("annot-color", &glib::Variant::from(color.as_str()));
         };
         self.set_action_state("annot-textmarkup-type", &glib::Variant::from(markup_type));
-
-        self.annot.replace(annot.cloned());
     }
 
     #[template_callback]
@@ -691,8 +691,12 @@ impl imp::PpsDocumentView {
             _ => panic!(),
         };
 
+        let show_zoom_fit_best = self.model.sizing_mode() != SizingMode::Automatic;
+        if self.zoom_fit_best_revealer.focus_child().is_some() && !show_zoom_fit_best {
+            self.view.grab_focus();
+        }
         self.zoom_fit_best_revealer
-            .set_reveal_child(self.model.sizing_mode() != SizingMode::Automatic);
+            .set_reveal_child(show_zoom_fit_best);
 
         action.change_state(&state.into());
 
